// ===============================================
// File:		      	GlutShell
// Description:		Glut-based OpenGL-Control
// Version 1:		  
// Current:		    
// Author(s):		  Thomas Kern
// ===============================================


//=====================================================================
//	INCLUDES
//=====================================================================
//
#include <math.h>
#include <stdio.h>

#include "objects.h"

//=====================================================================
// Messages
//=====================================================================
//
//---------------------------------------------------------------------------
static const char K_4 = '4';
static const char K_6 = '6';
static const char K_2 = '2';
static const char K_8 = '8';
static const char K_a = 'a';
static const char K_A = 'A';
static const char K_y = 'y';
static const char K_Y = 'Y';

static char M_QUIT[]     = {"Quit"};
static char M_PERSP[]    = {"gluPerspective"};
static char M_ORTHO[]    = {"glOrtho"};


//=====================================================================
// global Definitions
//=====================================================================
//
GLCoordSystem    *myCoord = NULL;


static boolean    readyToDraw = false;
static boolean 	 orthoView   = false;

static GLfloat    WindowX = 400.0, WindowY = 400.0;

static GLdouble  ViewingAngle, WindowAspect, nearPlane, farPlane;
static GLfloat   rotX, rotY, rotZ, moveZ[3];


//=====================================================================
// Change settings
//=====================================================================



//=====================================================================
// Initialize and Exit functions
//=====================================================================
//
// ViewingAngle: field of view angle(degrees), in y direction.
// WindowAspect: aspect ratio = field of view in x direction.
// The aspect ratio is the ratio of x (width) to y (height).
void SetProjection()
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

	 if( orthoView ) {
    // calculate the settings before
    GLdouble leftright = 5.0;
    GLdouble topdown   = leftright/WindowAspect;

    glOrtho(-leftright, leftright, // left, right borders
            -topdown,   topdown,   // bottom, top
            nearPlane, farPlane); // near, far
  }
  else {
    gluPerspective(ViewingAngle, WindowAspect,
		                nearPlane,    farPlane);
  }

  glMatrixMode(GL_MODELVIEW);
}

// --------------------------------------------------------------------
//
void Resize(int newWidth, int newHeight)
{
  if( newWidth && newHeight ) {
    WindowAspect = (GLfloat)newWidth / (GLfloat)newHeight;
    glViewport(0, 0, newWidth, newHeight);
    SetProjection();
  }
}

// --------------------------------------------------------------------
// new start
static void Reset()
{
  glClearColor(0.6,0.6,0.6,1.0);
  glEnable(GL_DEPTH_TEST);

  if(myCoord) {
  	 delete myCoord;
    myCoord = NULL;
  }
  myCoord = new GLCoordSystem();

  rotX = rotY = rotZ = moveZ[0] = moveZ[1] = 0.0;
  moveZ[2] = -16.0;
  ViewingAngle = 18.0;
  nearPlane =   0.5;
  farPlane  = 500.0;

  orthoView   = true;
  readyToDraw = true;
}

// --------------------------------------------------------------------
// exit program
static void Quit()
{
  if(myCoord) {
  	 delete myCoord;
    myCoord = NULL;
  }

  exit(EXIT_SUCCESS);
}


//=====================================================================
// control functions
//=====================================================================
//
// Keyboard events
static void myKeyboard(unsigned char key, int /*x*/, int /*y*/)
{
  if( readyToDraw ) {
    switch (key) {
    case K_4:
      rotY -= 5.0;
      break;
    case K_6:
      rotY += 5.0;
      break;
    case K_2:
      rotX += 5.0;
      break;
    case K_8:
      rotX -= 5.0;
      break;
    case K_a:
      moveZ[2] -= 0.5;
      break;
    case K_y:
      moveZ[2] += 0.5;
      break;
    case K_A:
      moveZ[2] -= 2.0;
      break;
    case K_Y:
      moveZ[2] += 2.0;
      break;
    case 27:
      Quit();
      break;
    }

    if( rotX > 360.0 ) rotX -= 360.0;
    if( rotX < 360.0 ) rotX += 360.0;
    if( rotY > 360.0 ) rotY -= 360.0;
    if( rotY < 360.0 ) rotY += 360.0;
    if( rotZ > 360.0 ) rotZ -= 360.0;
    if( rotZ < 360.0 ) rotZ += 360.0;

    glutPostRedisplay();
  }
}

//--------------------------------------------------------------------
// Mouse events
static void myMouse(int x, int y)
{
  if( readyToDraw && WindowX && WindowY ) {
    rotY = (GLfloat)( 180*(2*x/WindowX - 1) );
    rotX = (GLfloat)( 180*(2*y/WindowY - 1) );
    glutPostRedisplay();
  }
}

//--------------------------------------------------------------------
// what the menu items have to do
static void rightMenueSelect(int mode)
{
  switch (mode) {
  case  1:  orthoView = false;
  					 SetProjection();
            break;
  case  2:  orthoView = true;
  					 SetProjection();
            break;

  case 10:  Quit();
  }
  glutPostRedisplay();
}

//--------------------------------------------------------------------
// create menu
static void myGlutMenu(void)
{
  glutCreateMenu(rightMenueSelect);
  glutAddMenuEntry(M_PERSP,     1);
  glutAddMenuEntry(M_ORTHO,     2);
  glutAddMenuEntry(M_QUIT,     10);

  glutAttachMenu(GLUT_RIGHT_BUTTON);
}


//=====================================================================
// drawing
//=====================================================================
//
//
static void myDisplay(void)
{
  // readyToDraw should avoid to many calls of these routine
  if(readyToDraw) {
    readyToDraw = false;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    gluLookAt( 0.0, 0.0, 10.0,
    				    0.0, 0.0,  0.0,
               0.0, 1.0,  0.0 );

    glTranslatef(moveZ[0],moveZ[1],moveZ[2]);
    glRotatef(rotX, 1.0,0.0,0.0);
    glRotatef(rotY, 0.0,1.0,0.0);
    glRotatef(rotZ, 0.0,0.0,1.0);

		 if(myCoord) {
      glDisable(GL_CULL_FACE);
      myCoord->Draw();
      glEnable(GL_CULL_FACE);
    }

    // now we are ready to show the new scene
    glutSwapBuffers();
    readyToDraw = true;
  }
}


//=====================================================================
// main -- control
//=====================================================================
//
int main(int argc, char **argv)
{
  // what you can do with keys
  printf("\n======================================\n");
  printf("Direction: %c, %c, %c, %c\n", K_2, K_4, K_6, K_8);
  printf("Zoom     : %c, %c, %c, %c\n", K_a, K_A, K_y, K_Y);
  printf("remember you have a right mouse button...\n");
  printf("======================================\n");
  fflush(stdout);

  /* start of glut windowing and control functions */
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  glutInitWindowSize((int)WindowX, (int)WindowY);
  glutCreateWindow("thk: simple test");

  Reset();
  glutDisplayFunc(myDisplay);
  glutReshapeFunc(Resize); // Funktion aus GLsettings
  glutKeyboardFunc(myKeyboard);
  glutMotionFunc(myMouse);
  myGlutMenu();

  glutMainLoop();
  /* end of glut windowing and control functions */

  return 0;             /* ANSI C requires main to return int. */
}
